"""

    geomdl-cli - Run NURBS-Python (geomdl) from the command line
    Copyright (c) 2018 Onur Rauf Bingol <orbingol@gmail.com>

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.

"""

#
# NURBS evaluation and utility functions for geomdl-cli
#

import os
import os.path
from geomdl import __version__
from geomdl import NURBS
from geomdl import Multi
from geomdl import exchange
from geomdl.visualization import VisMPL


# File types allowed for importing
CLI_FILE_IMPORT_TYPES =  dict(
    cfg=exchange.import_cfg,
    conf=exchange.import_cfg,
    yaml=exchange.import_yaml,
    json=exchange.import_json,
)


def print_version():
    """Prints geomdl version"""
    print("geomdl version", __version__)


def generate_nurbs_from_file(file_name, delta, shape_idx, file_type=''):
    """Generates NURBS objects from supported file formats"""
    # Fix input types
    delta = float(delta)
    shape_idx = int(shape_idx)

    # Try to find file type from its extension
    if not file_type:
        fname, fext = os.path.splitext(file_name)
        file_type = fext[1:]

    ftype = file_type.lower()
    if ftype in CLI_FILE_IMPORT_TYPES:
        # Build NURBS object
        nurbs_objs = CLI_FILE_IMPORT_TYPES[ftype](file_name, delta=delta)

        # Return the shape
        if len(nurbs_objs) == 1:
            return nurbs_objs[0]

        if isinstance(nurbs_objs[0], NURBS.Curve):
            result = Multi.MultiCurve(nurbs_objs)
        else:
            result = Multi.MultiSurface(nurbs_objs)

        # Set the delta for multi shape objects
        if 0.0 < delta < 1.0:
            result.delta = delta

        if shape_idx >= 0:
            return result[shape_idx]
        return result
    else:
        raise RuntimeError("The input file type '" + str(file_type) + "' is not supported")


def build_vis(obj, **kwargs):
    """ Prepares visualization module for the input curve or surface.

    :param obj: input curve or surface
    :type obj: NURBS.Curve, NURBS.Surface, Multi.MultiCurve or Multi.MultiSurface
    :return: curve or surface updated with a visualization module
    """
    vis_config = VisMPL.VisConfig(**kwargs)
    if isinstance(obj, (NURBS.Curve, Multi.MultiCurve)):
        if obj.dimension == 2:
            obj.vis = VisMPL.VisCurve2D(vis_config)
        elif obj.dimension == 3:
            obj.vis = VisMPL.VisCurve3D(vis_config)
        else:
            raise RuntimeError("Can only plot 2- or 3-dimensional curves")

    if isinstance(obj, (NURBS.Surface, Multi.MultiSurface)):
        obj.vis = VisMPL.VisSurfTriangle(vis_config)

    return obj


def export_evalpts(obj, file_name, export_type):
    """ Prints the evaluated points on the screen and optionally exports them to a file.

    :param obj: input curve or surface
    :type obj: NURBS.Curve, NURBS.Surface, Multi.MultiCurve or Multi.MultiSurface
    :param file_name: name of the export file
    :type file_name: str
    :param export_type: type of the export file, e.g. txt, csv or vtk
    :type export_type: str
    """
    if export_type == "csv":
        exchange.export_csv(obj, file_name, point_type='evalpts')
    elif export_type == "txt":
        exchange.export_txt(obj, file_name, point_type='evalpts')
    elif export_type == "vtk":
        exchange.export_txt(obj, file_name, point_type='evalpts')
    else:
        if isinstance(obj, Multi.Abstract.Multi):
            sz = len(obj)
            for idx, opt in enumerate(obj.evalpts):
                for pt in opt:
                    line = ", ".join([str(p) for p in pt])
                    print(line)
                if idx != sz - 1:
                    print("---")
        else:
            for pt in obj.evalpts:
                line = ", ".join([str(p) for p in pt])
                print(line)


def export_nurbs(obj, file_name, export_type):
    """ Exports NURBS data in common CAD exchange formats.

    :param obj: input curve or surface
    :type obj: NURBS.Curve, NURBS.Surface, Multi.MultiCurve or Multi.MultiSurface
    :param file_name: name of the export file
    :type file_name: str
    :param export_type: type of the export file, e.g. cfg, obj, stl, ...
    :type export_type: str
    """
    type_maps = dict(
        cfg=exchange.export_cfg,
        yaml=exchange.export_yaml,
        json=exchange.export_json,
        smesh=exchange.export_smesh,
        obj=exchange.export_obj,
        stl=exchange.export_stl,
        off=exchange.export_off,
    )

    try:
        type_maps[export_type](obj, file_name)
    except KeyError:
        raise RuntimeError("The export method '" + str(export_type) + "' has not been implemented yet")
